% Value and policy functions are functions of:
% score, challenges left (by me, ignoring other side's challenging possibility),
% serving.
% Those will be calculated for given p_win_serve p_win_receive, p_cc, and the  
% functions cdf(p_mot) and p_aot(p_mot).
% See dynamicProb.pdf for details on how we solve the value functions.

% V is the value matrix (indices are scores,serving,challenges), p_mot is a
% similar matrix for policy function and DB is a list of points (according to
% the order of score::goBackwards) with their V and p_mot listed. The columns
% will be: sets_my sets_opp games_my games_opp points_my points_opp serving
% challenges V p_mot
function [V, p_mot, DB] = calcValues(p_win_serve,p_win_receive,p_cc,... 
                         cdf_of_p_mot, p_aot_of_p_mot, challengesInSetRule)  
                        % (above are the function handles)
    global suppress_warnings;

    %% Initialize
    % Define the value matrix

    % The matrix will hold a cell for each score, serving status and number of
    % challenges left. In a 3-set match there are 77 distinct scores for each
    % side (but less than 77X77 combinations because some are impossible like
    % 7:3 in sets). If 2 challenges are given per set, then there can be 0..3 
    % challenges possible (3 only in tiebreaks). If 3 are possible, then 0..4.
    % Thus, we need challenges-in-set + 2 cells to be able to accommodate a cell
    % for each challenge left given a score and a serving status.
    V = NaN(77,77,2,challengesInSetRule + 2);
    for serve = 0:1
        V(scr2ind(score(2,0,0,0,0,0,serve,challengesInSetRule,challengesInSetRule))) = 1;
        V(scr2ind(score(2,0,0,1,0,0,serve,challengesInSetRule,challengesInSetRule))) = 1;
        V(scr2ind(score(1,0,0,2,0,0,serve,challengesInSetRule,challengesInSetRule))) = 0;
        V(scr2ind(score(0,0,0,2,0,0,serve,challengesInSetRule,challengesInSetRule))) = 0;
    end
    
    % similar policies matrix
    p_mot = NaN(77,77,2,challengesInSetRule+2);

    % Deuces require value iteration. 
    % The first guess for the policy function is a vector of zeros.
    p_mot_vec_guess = zeros(6,1);

    % set maximum number of iterations for each deuce value calculation
    maxiter = 400;
    
    % set criterion for convergence (supnorm of the difference between two
    % consecutive p_mot calculations).
    delta_maximum = .01;    % if for the current delta we don't get convergence, 
                            % we will increase delta until it is more than 0.01
    
    p_win_by_serve = [p_win_receive, p_win_serve]';
    
    % this is for the endogenous values vector calculation
    p_win = kron(p_win_by_serve, ones(3,1));

    % choose  one of the scores from which if you'll go backwards you'll reach the 1.1 6:6 6-6
    % score (from which the calculation will start.
    s = score(2,0,0,0,0,0,0, challengesInSetRule, challengesInSetRule);
    
    if (challengesInSetRule == 2)
        DB = NaN(18048, 3);
    elseif (challengesInSetRule == 3)
        DB = NaN(23928, 3);
    else
        display 'Dont know how many rows there are in the database for this number of challenges in each set';
    end
        
    DB_index = 1;
    %% Start filling in the matrices
    % As long as we're not in 0.0 0:0 0-0:
    while ~isequal(goBackwards(s,1), s)
        s = goBackwards(s,1);
        if isdeuce(s) 
            s.challenges = 0;
%            display('Working on a deuce. Score is: ');
 %           show(s);

            chals = challengesInSetRule + istiebreak(s);
            
            % Start by calculating the values for the challenges=0 cells. They
            % do not depend on endogenous decisions because there are no
            % challenges left.
            % We have three scores (40-Adv, 40-40, Adv-40) and two serving
            % states, so we have 6 states for challenges=0. 
            % The two-column representation of the values is a 3X2 matrix of
            % values and the one-column representation of values is a 6X1 vector
            % (we need it for solving the system of equations)

            % calculating the values for challenges-left = 0
            % calculating the part before 
            M_exog = zeros(6);
            u_exog = zeros(6,1);
            for i=1:6
                temps = deuce2scr(s,i);
                % putting the A
                if scr2deuce(incrementMy(temps,1)) > 0 
                    M_exog(i,scr2deuce(incrementMy(temps,1))) = ...
                            p_win_by_serve(1 + temps.serving);
                else
                    u_exog(i) = p_win_by_serve(1 + temps.serving) * ...
                        V(scr2ind(incrementMy(temps, 1)));
                end
                
                % putting the B
                if scr2deuce(incrementOpp(temps,1,0)) > 0 
                    M_exog(i,scr2deuce(incrementOpp(temps,1,0))) = ...
                            1 - p_win_by_serve(1 + temps.serving);
                else
                    u_exog(i) = (1 - p_win_by_serve(1 + temps.serving)) * ...
                        V(scr2ind(incrementOpp(temps, 1, 0)));
                end
            end
            
            % calculate the values.
            v_exog = inv(eye(6)-M_exog)*u_exog;
            
            % updating the values in V.
            for i=1:6
                V(scr2ind(deuce2scr(s,i))) = v_exog(i);
                DB(DB_index, :) = [v_exog(i), NaN, NaN];
                DB_index = DB_index + 1;
            end

            % v_exog will be the guess of the values of the next
            % challenge level.

            for cur_chal = 1:chals
                % first guess is some increasing function, bounded by [0,1] of 
                % last challenge iteration's result
%                v_guess = sqrt(v_guess);
                s.challenges = cur_chal;
                
                delta = 1e-5;   % We will start with 10^-5 as the criterion for 
                                % convergence

                while delta < delta_maximum
                    for iter = 1:maxiter

                        % Create the current (challenge,serve)'s M matrix
                        % Calculating endogenous A B and C from the current p_mot
                        % guess. 
                        bounded_p_mot_vec_guess = min(1,max(0,p_mot_vec_guess));
                        if ((min(bounded_p_mot_vec_guess == p_mot_vec_guess) == 0) && (isempty(suppress_warnings) || (suppress_warnings == false)))
                            display('WARNING: p_mot is less than 0 or more than 1');
                            show(s);
                            display(sprintf('Iteration # %g',iter));
                            display(p_mot_vec_guess);
                        end

                        p_aot = p_aot_of_p_mot(bounded_p_mot_vec_guess);
                        G     = cdf_of_p_mot(bounded_p_mot_vec_guess);

                        As = p_win + ((1-p_win) .* p_cc) .* (1 - G) .* p_aot ;
                        Bs = (1 - p_win) .* (p_cc*G + (1 - p_cc)) ;
                        Cs = ((1 - p_win) * p_cc) .* (1 - G) .* (1 - p_aot);

                        M = zeros(6);
                        u = zeros(6,1);

                        for i=1:6
                            temps = deuce2scr(s,i);

                            % putting the A
                            if scr2deuce(incrementMy(temps,1)) > 0   % if endogenous value
                                M(i,scr2deuce(incrementMy(temps,1))) = As(i);
                            else                                     % if exogenous value
                                u(i) = As(i)*V(scr2ind(incrementMy(temps,1)));
                            end

                            % putting the B
                            if scr2deuce(incrementOpp(temps,1,0)) > 0 % if endogenous value
                                M(i,scr2deuce(incrementOpp(temps,1,0))) = Bs(i);
                            else                                      % if exogenous value
                                u(i) = Bs(i)*V(scr2ind(incrementOpp(temps,1,0)));
                            end

                            % putting the C (value exogenous always b/c calculated in previous round)
                            u(i) = u(i) + Cs(i)*V(scr2ind(incrementOpp(temps,1,1)));
                        end

                        % ready to calculate this iteration's v:
                        v_next = inv(eye(size(M))-M)*u;                

                        p_mot_vec = zeros(6,1);
                        % calculate implied p_mot from the last-guessed value vector
                        for i=1:6
                            scrOfPmot = deuce2scr(s,i);

                            if (scr2deuce(incrementMy(scrOfPmot,1)) > 0)
                                v_p_1 = v_next(scr2deuce(incrementMy(scrOfPmot,1)));
                            else
                                v_p_1 = V(scr2ind(incrementMy(scrOfPmot,1)));
                            end

                            if (scr2deuce(incrementOpp(scrOfPmot,1,0)) > 0)
                                v_m_1 = v_next(scr2deuce(incrementOpp(scrOfPmot,1,0)));
                            else
                                v_m_1 = V(scr2ind(incrementOpp(scrOfPmot,1,0)));
                            end

                            % the challenges-1 value is definitely not endogenous in
                            % this round (was already calculated before).
                            v_m_1_c_1 = V(scr2ind(incrementOpp(scrOfPmot,1,1)));

                            p_mot_vec(i) = (v_m_1 - v_m_1_c_1) / (v_p_1 - v_m_1_c_1);
                        end

%                         if s.mySets == 1 && s.oppSets == 0 && ...
%                                 s.myGames == 5 && s.oppGames==0 
%                             display(p_mot_vec');
%                             display(p_mot_vec'-p_mot_vec_guess');
%                         end
                            
                        if max(abs(p_mot_vec_guess - p_mot_vec)) < delta
                            converged = true;
                            break;
                        else
                            converged = false;
                            p_mot_vec_guess = p_mot_vec;
                        end
                    end
                    if converged
                        break;
                    else
                        if (~(suppress_warnings == true))
                            display(['No convergence for delta = ' num2str(delta)]);
                            display(p_mot_vec_guess);
                        end
                        delta = delta*5;
                        if (~(suppress_warnings == true))
                            display(['Restarting with delta = ' num2str(delta)]);
                        end
                    end
                end
                if converged
%                     if (~isreal(v_next) || ~isreal(p_mot_vec)) 
%                         display('NON REAL VALUE OR POLICY!!');
%                         display(v_next);
%                         display(p_mot_vec);
%                     end
                    % Update V and p_mot
                    for i=1:6
                        V(scr2ind(deuce2scr(s,i))) = v_next(i);
                        p_mot(scr2ind(deuce2scr(s,i))) = p_mot_vec(i);
                        DB(DB_index, :) = [v_next(i), p_mot_vec(i), delta];
                        DB_index = DB_index + 1;
                    end
                else
                    display ('Deuce value iteration failed to converge!');
                    show(s);
                    V = -999;
                    return; 
                end
            end
        else
            % In a regular point we have the values of the following points, so
            % we can immediately calculate p_mot and V
            % For all challenges from 2 down to 0 or from 3 down to 0 (depending
            % on whether it's a tiebreak).
            for servstat = 0:1 
                cur_p_win = p_win_by_serve(servstat + 1);
                s.serving = servstat;
                
                % for challenges from 2 (or 3) to 1 (challenge 0 will be dealt
                % with later
                for chals = (challengesInSetRule+istiebreak(s)):-1:1
                    s.challenges = chals;
                    
                    % This is equation (5) from the dynamicProb.pdf:
                    % cur_p_mot = [V(s-1,c) - V(s-1,c-1)] / [V(s+1,c) - V(s-1,c-1)]
                    cur_p_mot = (V(scr2ind(incrementOpp(s,1,0))) - ...
                                    V(scr2ind(incrementOpp(s,1,1)))) / ...
                                (V(scr2ind(incrementMy(s,1))) - ...
                                    V(scr2ind(incrementOpp(s,1,1))));
                     
                    error_flag = 0;
                    if (cur_p_mot < 0 || cur_p_mot > 1)
                        if (~(suppress_warnings == true))
                            display('ERROR (warning): P_mot is less than 0 or more than 1');
                            show(s);
                            display(cur_p_mot);
                            display(['V(s-1,c) = ' num2str(V(scr2ind(incrementOpp(s,1,0))))]);
                            display(['V(s-1,c-1) = ' num2str(V(scr2ind(incrementOpp(s,1,1))))]);
                            display(['V(s+1,c) = ' num2str(V(scr2ind(incrementMy(s,1))))]);
                        end
                        orig_cur_p_mot = cur_p_mot;
                        cur_p_mot = max(min(cur_p_mot, 1), 0);
                        error_flag = 1;
                    end
                    
                    p_mot(scr2ind(s)) = cur_p_mot;
                    
                    % bound p_mot to be between 0 and 1 (for the G and p_aot
                    % calculations).
                    cur_p_mot = min(1,max(0,cur_p_mot));
                    
                    p_aot = p_aot_of_p_mot(cur_p_mot);
                    G     = cdf_of_p_mot(cur_p_mot);
                
                    A = cur_p_win + (1-cur_p_win) * p_cc * (1 - G) * p_aot;
                    B = (1 - cur_p_win) * (p_cc*G + (1 - p_cc));
                    C = (1 - cur_p_win) * p_cc * (1 - G) * (1 - p_aot);
                    
                    V(scr2ind(s)) = A * V(scr2ind(incrementMy(s,1))) + ...
                                    B * V(scr2ind(incrementOpp(s,1,0))) + ...
                                    C * V(scr2ind(incrementOpp(s,1,1)));
                    
%                     if (~isreal(V(scr2ind(s))) || ~isreal(cur_p_mot)) 
%                         display('NON REAL VALUE OR POLICY!!');
%                         display(v_next);
%                         display(cur_p_mot);
%                     end
                    if (error_flag)
                        DB(DB_index, :) = [V(scr2ind(s)), cur_p_mot, orig_cur_p_mot];
                    else
                        DB(DB_index, :) = [V(scr2ind(s)), cur_p_mot, NaN];
                    end
                    DB_index = DB_index + 1;
                end
                
                s.challenges = 0;
                V(scr2ind(s)) = cur_p_win * V(scr2ind(incrementMy(s,1))) + ...
                                (1 - cur_p_win) * V(scr2ind(incrementOpp(s,1,0)));
                DB(DB_index, :) = [V(scr2ind(s)), NaN, NaN];
                DB_index = DB_index + 1;
            end
        end
    end
        
end