% This function is used for the robustness checks in the paper. Unlike the
% trySimulateMatchInChunksFunc, it calculates values for ALL the
% p_win_point_if_serve X p_win_point_if_receive grid, instead of just one column
% (because it does the estimation of the policy in the data as a part of the
% program). trySimulateMatchInChunksFunc assumes we have the deviation of the
% data from the model given, and calculates the table for chunks of the grid
% (each chunk is a given p_win_if_serve value).

% This function calculates the value function for given p_cc, F_alpha and
% intercept (that define the technological assumptions) for a grid. then
% interpolates in the data the policies to estimate how much players differ in
% reality than the implied policy. After getting the estimated number from the
% data, it translates the optimal policy to the observed one. It also calculates
% the value of a match given other strategies of challenging and compares them
% in a table (spit out into filenames with the "filename" parameter in them).
function outputTable = trySimulateMatchFunc(filename, p_cc, F_alpha, intercept)
    % This is the place where we set the technological assumptions about the
    % distribution of p_mot and p_aot(p_mot) as a result. We assume
    % p_mot~Beta(alpha,beta).
    % To have the Paot function cut the y-axis at "intercept", we need to define
    % the beta parameter of the cdf F_beta=F_alpha*(1/intercept-1)
    F_beta = F_alpha*(1/intercept-1)

    % Loading the data from file
    temp = importdata('point_data_after_importance_round3_for_matlab.txt');
    data = dataset(cat(2,{temp.data}, temp.colheaders));
    clear temp;
    
    p_win_serve_grid = .1:.2:.9;
    p_win_receive_grid = .1:.2:.9;

    p_aot_handle = @(p) p_aot(p,F_alpha,F_beta);
    G_handle = @(p) betainc(p,F_alpha,F_beta);

    outputTable = NaN(26,9);
    
    tic
    [p_mot_mega2 V_mega2] = tryCalcValuesFunc(true, p_cc, F_alpha, F_beta);
    toc
  
    % DEBUG (to save time calculating value and policy again and again, if the
    %problem is not there)
    % save temp2_p_mot_V_mega2 p_mot_mega2 V_mega2;
    % load  temp2_p_mot_V_mega2
    
    display 'Done calculating optimal strategy. Looking at data to see how it matches';
    
    [p_aot_in_data_handle, figuresTable] = ...
            compareData2Optimal(data, p_mot_mega2, F_alpha, F_beta);

    figuresTable = set(figuresTable, 'Description', ...
                        sprintf('Data vs. model for p_cc=%g, F_alpha=%g, and intercept=%g',...
                                p_cc, F_alpha, intercept));
                            
    save(strcat('figures_data_',filename), 'figuresTable');
    
    % THIS IS WHERE YOU CONFIGURE THE TRANSFORMATION FROM THE OPTIMAL TO THE
    % SUBOPTIMAL POLICIES.
    % opt_subopt_translation should be a function handle that takes an (optimal)
    % p_mot -- for which we have a mapping from the state which is observable --
    % to a (suboptimal) p_mot that will yield the same p_aot that we see in the
    % data.
    % SEE Simulating optimal and suboptimal challenging strategies - Overview.docx
    % FOR DETAILS.
    % If we don't get it, we assume a linear transformation

    p_space = 0:0.01:1;
    opt_subopt_trans_vec = interp1(p_aot_handle(p_space), ...
                                    p_space, ...
                                    p_aot_in_data_handle(p_space));
                       
    % If the data Paot(p) is lower than the model's Paot(p), then our 
    % interpolation of Paot_model^-1(Paot_data(0)) is negative, because there is
    % no x for which Paot_model(x)<Paot_model(0). Thus, we set the strategies to
    % be the same at this point (we can't challenge more than "always")
    opt_subopt_trans_vec(isnan(opt_subopt_trans_vec)&p_aot_in_data_handle(p_space) < ...
            p_aot_handle(p_space))=0;
    
    subopt_translation = @(p) interp1(p_space, opt_subopt_trans_vec, p);

    plot(p_space,p_aot_handle(p_space),...
         p_space,p_aot_in_data_handle(p_space), ...
         p_space,subopt_translation(p_space)-p_space);
    title('Calculating values for optimal and suboptimal (as in data) strategies in new games');
    legend('p_aot(p_mot) (or G(y)) in model','p_aot(p_mot) (or G(y)) in data (given)', 'optimal to suboptimal translation');

    saveas(gcf,strcat('opt_subopt_graph_',filename,'.png'));
    
    % HERE we calculate p_chal_random: the probability to challenge when we use 
    % a "challenge randomly" strategy. 
    % We take the average success rate we see in the data, use the inverse of 
    % the P_aot(P_mot) function to get P_mot (or q* in the paper). 
    % 1-G(q*) is what we set as p_chal (F(q*) in the paper).
    avg_success_in_data = mean(data.challenge_won);
    
    p_chal_random = 1-G_handle(interp1(p_aot_handle(p_space), ...
                                    p_space, ...
                                    avg_success_in_data));
    
    display('Calculating the value of playing a match under different, non-optimal, strategies');
    
    for p_serve_i = 1:length(p_win_serve_grid)
        p_win_if_serve = p_win_serve_grid(p_serve_i)
        for p_receive_i = 1:length(p_win_receive_grid)
            p_win_if_receive = p_win_receive_grid(p_receive_i)

            tableRowIndex = (p_serve_i-1)*5 + p_receive_i;
            
            % The optimal p_mot as given by the dynamic programming problem solution
            optimal_p_mot_matrix = p_mot_mega2{1}{p_serve_i, p_receive_i};

            % Always-challenge policy, in other words cutoff for challenging is zero
            always_challenge_p_mot_matrix = zeros(size(optimal_p_mot_matrix));

            % Never-challenge policy, in other words cutoff for challenging is one
            % (we want to see if it's equal to the Passerman model assuming no
            % challenges possible).
            never_challenge_p_mot_matrix = ones(size(optimal_p_mot_matrix));

            % The value from playing optimally (already calculated)
            tic
            optV = V_mega2{1}{p_serve_i, p_receive_i};   % already calculated (actually recalculating will give slightly different values due to rounding error)
            toc

            % if the value function solution did not converge in one of the deuces,
            % we will have a NaN for the first point, because we did not reach it.
            % In any case, we can't really get the value of an optimal or a
            % suboptimal strategy. The always-challenge or never-challenge are,
            % however, possible to calculate because we know the p_mot's.
            s = score(0,0,0,0,0,0,1,2,2);
            if (~isnan(optimal_p_mot_matrix(scr2ind(s))))
                % The suboptimal p_mot as seen in the data, translated to the
                % appropriate "optimal" p_mot that will yield the same winning
                % proportions that we see in the data.
                optimal_p_mot_matrix(~isnan(optimal_p_mot_matrix)) = min(max(optimal_p_mot_matrix(~isnan(optimal_p_mot_matrix)),0),1);
                suboptimal_p_mot_matrix = subopt_translation(optimal_p_mot_matrix);
                
                tic          
                suboptV = calcValuesFromKnownPmots(p_win_if_serve,p_win_if_receive,p_cc,... 
                         G_handle, p_aot_handle, suboptimal_p_mot_matrix, 2);
                toc
                outputTable(tableRowIndex, 5) = optV(scr2ind(s));
                outputTable(tableRowIndex, 6) = suboptV(scr2ind(s));
            end
            
            % We used to calculate an "always challenge" strategy where every close call is
            % being challenged. This is not very interesting (even though close to the
            % optimal for low p_cc's). What we do now, instead, is to imagine that the
            % player challenges randomly: whenever there's a close call, he flips a coin
            % with probability p_chal. This probability is calculated to yield a lower
            % incidence of challenging. 
            % After we get p_chal, we restate p_cc as the product of the original p_cc and
            % p_chal and set the policy to be always challenge. This way, looking forward, a
            % player knows that he will challenge the next point with probability
            % p_cc*p_chal_random.
            tic
            randomV = calcValuesFromKnownPmots(p_win_if_serve,p_win_if_receive,p_cc*p_chal_random,... 
                 G_handle, p_aot_handle, always_challenge_p_mot_matrix, 2);
            toc

            tic          
            neverV = calcValuesFromKnownPmots(p_win_if_serve,p_win_if_receive,p_cc,... 
                     G_handle, p_aot_handle, never_challenge_p_mot_matrix, 2);
            toc  

            outputTable(tableRowIndex, 4) = neverV(scr2ind(s));
            outputTable(tableRowIndex, 7) = randomV(scr2ind(s));

            outputTable(tableRowIndex, 8) = (outputTable(tableRowIndex, 6) - outputTable(tableRowIndex, 4)) / ...
                                  (outputTable(tableRowIndex, 5) - outputTable(tableRowIndex, 4));
            outputTable(tableRowIndex, 9) = (outputTable(tableRowIndex, 7) - outputTable(tableRowIndex, 4)) / ...
                                  (outputTable(tableRowIndex, 5) - outputTable(tableRowIndex, 4));

            outputTable(tableRowIndex, 1) = p_win_if_serve;
            outputTable(tableRowIndex, 2) = p_win_if_receive;
            outputTable(tableRowIndex, 3) = ...
                    size(data(data.p_win_pt_receive >   round(p_win_if_receive*10)/10-.1 & ...
                              data.p_win_pt_receive <=  round(p_win_if_receive*10)/10+.1 & ...
                              data.p_win_pt_serve   >   round(p_win_if_serve*10)/10-.1 & ...
                              data.p_win_pt_serve   <=  round(p_win_if_serve*10)/10+.1,1),1) / ...
                    size(data,1);
                
            display('The output for the optimal/suboptimal comparison table is...');
            outputTable(tableRowIndex, :)
        end
    end
    
    sumover = ~isnan(outputTable(:,3)) & ~isnan(outputTable(:,5));
    
    outputTable(length(p_win_serve_grid)*length(p_win_receive_grid)+1,3) = ...
                    sum(outputTable(:,3));
                
    % now get the weighted sum of the other columns (weighted by the proportion
    % of observations around gridpoint).
    % note that we reweight column 3 with the frequencies because we ignore
    % state
    outputTable(length(p_win_serve_grid)*length(p_win_receive_grid)+1,4:9) = ...
                    (outputTable(sumover,3)/sum(outputTable(sumover,3)))' * ...
                    outputTable(sumover,4:9);
                
    save(strcat('opt_subopt_cmp', filename),'outputTable');
end
