
// Build a matrix that will represent the game tree:
// (transposed):
//
//          /
//         /  
//        /\  /
//       /  \/
//      /\  /\  /   / 
//     /  \/  \/   /  
//    /\  /\  /\  /\  
//   /  \/  \/  \/  \ ... 
//   \  /\  /\  /\  / 
//    \/  \/  \/  \/ 
//     \  /\  /\   \ 
//      \/  \/  \   \ 
//       \  /\  
//        \/  \
//         \
//          \
//
// Each node is a game score starting with 0:0 in the root, then every win goes up in the tree
// and every loss goes down.
// When we transpose the tree, the columns are advantage (from -4 to 4, but we normalize the 
// matrix indices to start with 1 instead of -4). The rows are the number of points played until
// we reached this stage (starting at 0:0). They also normalize to start from 1 instead of 0.
//
// Matrix will look like this:
//            1      2      3      4      5      6      7      8      9
//     +----------------------------------------------------------------+
//   1 |     0      0      0      0      1      0      0      0      0  |
//   2 |     0      0      0     .4      0     .6      0      0      0  |
//   3 |     0      0    .16      0    .48      0    .36      0      0  |
//   4 |     0   .064      0   .288      0   .432      0   .216      0  |
//   5 |  .026      0   .154      0   .346      0   .346      0    .13  |
//   6 |     0   .061      0    .23      0   .346      0   .207      0  |
//   7 |     0      0   .092      0   .276      0   .207      0      0  |
//   8 |     0      0      0      0      0      0      0      0      0  |
//   9 |     0      0   .044      0      0      0     .1      0      0  |
//     +----------------------------------------------------------------+
//   Where each cell above has the probability to play the score below
//
//          1      2      3      4      5      6      7      8      9
//     +----------------------------------------------------------------+
//   1 |                               0:0                              |
//   2 |                        0:1           1:0                       |
//   3 |                 0:2           1:1           2:0                |
//   4 |          0:3           1:2           2:1           3:0         |
//   5 |   0:4           1:3           2:2           3:1           4:0  |
//   6 |          1:4           2:3           3:2           4:1         |
//   7 |                 2:4            D            4:2                |
//   8 |                                                                |
//   9 |                 L.D                         W.D                |
//     +----------------------------------------------------------------+
// (where D is Deuce (x:x s.t x>=3), L.D is Lost Deuce and W.D is Won Deuce)
// The probability to win, therefore, is the sum of cells (5,9), (6,8), (7,7)
// and (9,7).

mata:
struct gameStruct {
	real matrix probs
	real scalar Pwin , EpointsLeft
}

// For a set, we also want the probability that the set ended after 
// to win after an even number of games (and start the next set with 
// the same side serving)
struct setStruct {
	real matrix probs
	real scalar Pwin , EpointsLeft
	real scalar Pwin_n_serve, Pwin_n_receive, Plose_n_serve, Plose_n_receive
}

struct matchStruct {
	real scalar Pwin
	real matrix probs
}

//     myPoints and oppPoints - the score of the game when you want to do the forward looking 
//		probability to arrive to all subsequent points calculation. Technically, it will determine 
//      in which node we will have the Pr[point] == 1
//     Pwin - is the probability of me to win (the player for which myPoints were specified)
struct gameStruct scalar gameCalc(real scalar myPoints, real scalar oppPoints, real scalar PwinPoint) {	
	struct gameStruct scalar returnMe
	
	game  	  = J(9, 9,0)
	orig_adv = myPoints - oppPoints
	orig_points = myPoints + oppPoints
	
	// Correct points to 3:2 or 3:3 if they are 17:16 or 15:15
	if (abs(orig_adv) <= 1 && orig_points > 6) {
		if (abs(orig_adv) == 1) orig_points = 5
		else orig_points = 6
	}
	else if (abs(orig_adv) == 2 && orig_points > 6) {  // Same if we have 18:16 (it's a sure-win)
		orig_points = 8  
	}
	else if (abs(orig_adv) > 2 && (myPoints > 4 || oppPoints > 4)) {
		printf("\nError in input point score for game (%g:%g)\n", myPoints, oppPoints)
		returnMe.probs = game
		returnMe.Pwin = .
		returnMe.EpointsLeft = .
		
		return (returnMe)
	}
	
	// Setting the root cell. 
	// Remember - we add 5 to the advantage when we refer to a cell in the matrix, because indexes
	// start in 1 instead of -4.
	game[orig_points + 1, orig_adv + 5] = 1
	
	// We will go in each line (number of points played) and for each node we will take the source node
	// from the previous line (the node from which we can arrive to this node) and multiply by the probability
	// to arrive to this point (either win or lose). From a (adv-1) advantage to adv, we need to win (to go from 
	// -1 to 0 or from 2 to 3) and from a (adv+1) advantage to adv, we need to lose (from 2 to 1 or -3 to -4).
	// When both are possible (to get to 2:1 we can either win after a 1:1 or lose after a 2:0), we sum the
	// probabilities because the events (score in the previous point) are disjoint.
	for(points = orig_points + 1 ; points <= 6 ; points++) {
		for(adv=-4 ; adv <= 4 ; adv++) {
			// When do we have two sources for the same point?
			// (1) Whenever adv is 0 (either in simple game or advantages stage)
			// (2) When |adv| is 1 and the number of points played is 5 or less (2:3 comes from 2:2 or 1:3)
			// (3) When |adv| is 2 and the number of points played is 4 or less (1:3 comes from 0:3 or 1:2)
			//   ==> (2) and (3) can be united to one condition of |adv| + points <= 6
			// We also need to make sure we don't fill in the cells above the top chevron (advantage can't be more
			// than total points played, or in other words - |column-midColumn| can't be bigger than row)
			if (adv == 0 | ((abs(adv) + points <= 6) && (abs(adv) <= points))) {
				game[points+1, 5 + adv] = 	game[points , 5 + adv - 1] * PwinPoint + ///
											game[points , 5 + adv + 1] * (1-PwinPoint)
			}
			// When do we have one source for the same point?
			// All other possible points, and then the source is from the same side (if we're in a positive adv, then
			// it's from the adv-1 and then we won. if we're in a negative adv, then adv+1 and then lost)
			else if (abs(adv)+points <= 8 | abs(adv) <= 1 | abs(adv) == 2 && mod(points, 2) == 0) {
				if (adv > 0) {
					game[points+1, 5 + adv] = game[points , 5 + adv - 1] * PwinPoint
				}
				else {
					game[points+1, 5 + adv] = game[points , 5 + adv + 1] * (1-PwinPoint)
				}
			}
		}
	}
	
	// We might have already got 5:3, so no need to update the 1 and 0 there.
	if (orig_points < 8) {
		// Add the endpoints for winning after a 3:3 (40:40)
		// A final win that makes 3:3 become 5:3 (to me) is equal to
		// Padvwin = PwinPoint^2 / (PwinPoint^2 + (1-PwinPoint)^2)
		// Thus, cell (9,3) will be equal to cell (7,5) * (1-Padvwin)
		// and then cell (9,7) will be equal to cell (7,5) * Padvwin
		game[9,3] = game[7,5] * (1 - PwinPoint^2 / (PwinPoint^2 + (1-PwinPoint)^2))
		game[9,7] = game[7,5] *   PwinPoint^2 / (PwinPoint^2 + (1-PwinPoint)^2)
	}
	
	// Return the probability matrix, probability to win the whole game and
	// expected points left
	returnMe.probs = game
	returnMe.Pwin = game[5,9] + game[6,8] + game[7,7] + game[9,7]
	returnMe.EpointsLeft = (game[5,1] + game[5,9]) * 4 + ///
							(game[6,2] + game[6,8]) * 5 + ///
							(game[7,3] + game[7,7]) * 6 + ///
							(game[9,3] + game[9,7]) * (6 + 2*(1 / (PwinPoint^2 + (1-PwinPoint)^2))) ///
							- orig_points
	
	return(returnMe)
}

//     myPoints and oppPoints - the score of the game when you want to do the forward looking 
//		probability to arrive to all subsequent points calculation. Technically, it will determine 
//      in which node we will have the Pr[point] == 1
//     PwinServ and PwinRecv - is the probability of me to win if serving and if receiving (for the player 
//		for which myPoints were specified)
//	   serving - is 1 if I am serving this point or 0 otherwise.
struct gameStruct scalar tiebreakCalc(	real scalar myPoints, real scalar oppPoints, ///
							real scalar PwinPointServ, real scalar PwinPointRecv, ///
							real scalar serving) {
	struct gameStruct scalar returnMe
	
	game  	  = J(15, 15,0)
	orig_adv = myPoints - oppPoints
	orig_points = myPoints + oppPoints
	
	// Correct points to 3:2 or 3:3 if they are 17:16 or 15:15
	if (abs(orig_adv) <= 1 && orig_points > 12) {
		if (abs(orig_adv) == 1) orig_points = 11
		else orig_points = 12
	}
	else if (abs(orig_adv) == 2 && orig_points > 12) {  // Same if we have 18:16 (it's a sure-win)
		orig_points = 14
	}
	else if (abs(orig_adv) > 2 && (myPoints > 7 || oppPoints > 7)) {
		printf("\nError in input point score for tiebreak (%g:%g)\n", myPoints, oppPoints)
		returnMe.probs = game
		returnMe.Pwin = .
		returnMe.EpointsLeft = .
		
		return (returnMe)
	}
		
	
	// Setting the root cell. 
	// Remember - we add 8 to the advantage when we refer to a cell in the matrix, because indexes
	// start in 1 instead of -7
	game[orig_points + 1, orig_adv + 8] = 1
	
	// We will go in each line (number of points played) and for each node we will take the source node
	// from the previous line (the node from which we can arrive to this node) and multiply by the probability
	// to arrive to this point (either win or lose). From a (adv-1) advantage to adv, we need to win (to go from 
	// -1 to 0 or from 2 to 3) and from a (adv+1) advantage to adv, we need to lose (from 2 to 1 or -3 to -4).
	// When both are possible (to get to 2:1 we can either win after a 1:1 or lose after a 2:0), we sum the
	// probabilities because the events (score in the previous point) are disjoint.
	for(points = orig_points + 1 ; points <= 12 ; points++) {
		// Determine who is serving for the probabilities to win
		// Serving goes this way:
		// Server: 			A , B , B , A , A , B , B , A , A , B , B , ...
		// points played     1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10, 11, ...
		// points-2: 		  -1 , 0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , ...
		// (we always look at the path from the previous point to this point, so total points up to the last one (points-1)
		//  is what matters for the server of the following point)
		// mod(points-2,4):  3 , 0 , 1 , 2 , 3 , 0 , 1 , 2 , 3 , 0 , 1
		// So if (mod(points-2, 4) < 2 and mod(orig_points-1, 4) < 2) or 
		//		 (mod(points-2, 4) >= 2 and mod(orig_points-1, 4) >= 2)
		//     then we have the same server. Otherwise we have the other side serving
		if ((mod(points-2, 4) < 2 && mod(orig_points-1, 4) < 2) | ///
			(mod(points-2, 4) >= 2 && mod(orig_points-1, 4) >= 2)) {
			PwinPoint = PwinPointServ * serving + PwinPointRecv * (1 - serving)
		}
		else {
			PwinPoint = PwinPointServ * (1 - serving) + PwinPointRecv * serving
		}
		
		for(adv=-7 ; adv <= 7 ; adv++) {
			// When do we have two sources for the same point?
			// (1) Whenever adv is 0 (either in simple game or advantages stage)
			// (2) When |adv| is 1 and the number of points played is 13 or less (6:5 comes from 5:5 or 6:4,
			//																		but 7:6 can only come from 
			//																		6:5, so it doesn't count).
			// (3) When |adv| is 2 and the number of points played is 12 or less (4:6 comes from 3:6 or 4:5)
			// ... 
			// (k) When |adv| is (k-1) and the number of points played is (15-k) or less
			// ...
			// (6) When |adv| is 5 and the number of points played is 6 or less (6:1 comes from 5:1 or 6:0)
			//   ==> (2) through (6) can be united to one condition of |adv| + points <= 14
			// We also need to make sure we don't fill in the cells above the top chevron
			if (adv == 0 | ((abs(adv) + points <= 12) && (abs(adv) <= points))) {
				game[points+1, 8 + adv] = 	game[points , 8 + adv - 1] * PwinPoint + ///
											game[points , 8 + adv + 1] * (1-PwinPoint)
			}
			// When do we have one source for the same point?
			// All other possible points, and then the source is from the same side (if we're in a positive adv, then
			// it's from the adv-1 and then we won. if we're in a negative adv, then adv+1 and then lost)
			else if (abs(adv)+points <= 14 | abs(adv) <= 1 | abs(adv) == 2 && mod(points, 2) == 0) {
				if (adv > 0) {
					game[points+1, 8 + adv] = game[points , 8 + adv - 1] * PwinPoint
				}
				else {
					game[points+1, 8 + adv] = game[points , 8 + adv + 1] * (1-PwinPoint)
				}
			}
		}
	}
	
	// We might have already got 8:6, so no need to update the 1 and 0 there.
	if (orig_points < 14) {
		// Add the endpoints for winning after a 6:6
		// A final win that makes 6:6 become 8:6 (to me) is equal to
		// Padvwin = PwinPointServ*PwinPointRecv / (PwinPointServ*PwinPointRecv + (1-PwinPointServ)(1-PwinPointRecv))
		// Thus, cell (15,6) will be equal to cell (13,8) * (1-Padvwin)
		// and then cell (15,10) will be equal to cell (13,8) * Padvwin

		game[15,6]  = game[13,8] * ///
					(1 - PwinPointServ*PwinPointRecv / (PwinPointServ*PwinPointRecv + (1-PwinPointServ)*(1-PwinPointRecv)))
		game[15,10] = game[13,8] * ///
					PwinPointServ*PwinPointRecv / (PwinPointServ*PwinPointRecv + (1-PwinPointServ)*(1-PwinPointRecv))
	}

		// Calculating expectations, in a loop, but basically
	/* like the  (in the game case):
							(game[5,1] + game[5,9]) * 4 + ///
							(game[6,2] + game[6,8]) * 5 + ///
							(game[7,3] + game[7,7]) * 6 + ///
							(game[9,3] + game[9,7]) * (6 + (E(deuce_points)))) <== will be added after loop. */
	
	expectation = 0 ;
	winningProb = 0;
	for(i=7; i<=12 ; i++) {
		expectation = expectation + (game[i+1,-6+i] + game[i+1,22-i]) * i
		winningProb = winningProb + game[i+1,22-i]
	}
	
	expectation = expectation + (game[15, 6] + game[15, 10]) * ///
				(12 + 2*(1/(PwinPointServ*PwinPointRecv + (1-PwinPointServ)*(1-PwinPointRecv))))
	
	// finally, remove orig_points (if we didn't start in 0-0)
	expectation = expectation - orig_points
	
	winningProb = winningProb + game[15,10]
	
	// Return the probability matrix, probability to win the whole game and
	// expected points left
	returnMe.probs = game
	returnMe.Pwin = winningProb
	returnMe.EpointsLeft = expectation
	
	return(returnMe)
}


struct setStruct scalar function setCalc(	real scalar myGames, real scalar oppGames, ///
							real scalar myPoints, real scalar oppPoints, ///
							real scalar PwinPointServ, real scalar PwinPointRecv, ///
							real scalar serving, |real scalar debug) {
	struct gameStruct scalar genGameServ, genGameRecv, origGame, tieBreak
	struct setStruct scalar returnMe
	
	if (args() < 8) debug = 0;
		
	mySet  	  = J(14, 13,0)
	orig_adv = myGames - oppGames
	orig_games = myGames + oppGames

	// Get atoms' (games & tiebreak) probabilities
	genGameServ = gameCalc(0, 0, PwinPointServ)
	genGameRecv = gameCalc(0, 0, PwinPointRecv)

	// DEBUG
	if (debug) {
		printf("Probability to win a game starting 0:0 serving is %g\n", round(genGameServ.Pwin, 0.001))
		printf("Probability to win a game starting 0:0 receiving is %g\n", round(genGameRecv.Pwin, 0.001))
	};

	errFlag = 0
	
	if (myGames == 6 && oppGames == 6) { // tiebreaker
		origGame = tieBreak = tiebreakCalc(myPoints, oppPoints, PwinPointServ, PwinPointRecv, serving)
		if (origGame.Pwin == .) errFlag = 1;		
	}
	else if (max((myGames,oppGames)) > 7 || ///
			(max((myGames,oppGames)) == 7 && (min((myGames, oppGames)) < 5) || min((myGames, oppGames)) > 6)) {
		printf("... Error in input point score for set (%g:%g)\n", myPoints, oppPoints)
		errFlag = 1;
	}
	else { // Valid score, not a tiebreak
		origGame = gameCalc(myPoints, oppPoints, serving*PwinPointServ + (1-serving)*PwinPointRecv)
		if (origGame.Pwin == .) errFlag = 1
		else {
			// Serving in the TB is the same as this game if total points up to this game
			// are even (player who served the first game when myGames+oppGames=0,
			//                          or the third game when myGames+oppGames=2, ...)
			if (mod(myGames + oppGames, 2) == 0) 
				tieBreak = tiebreakCalc(0, 0, PwinPointServ, PwinPointRecv, serving);
			else
				tieBreak = tiebreakCalc(0, 0, PwinPointServ, PwinPointRecv,1-serving);
		}
	} 
	
	if (errFlag) {
		returnMe.probs = mySet
		returnMe.Pwin = .
		returnMe.EpointsLeft = .
		returnMe.Pwin_n_serve = returnMe.Pwin_n_receive = returnMe.Plose_n_receive = ///
			returnMe.Plose_n_serve = .
		return (returnMe)
	}

	// DEBUG
	if (debug) {
		printf("Probability to win cur game starting %g (me) - %g (opp) (serving==%g) is %g\n", ///
					myPoints, oppPoints, serving, round(origGame.Pwin, 0.00001))
		round(origGame.probs,0.0001)
		
		printf("Probability to win tiebreak is %g\n", round(tieBreak.Pwin, 0.00001))
		round(tieBreak.probs,0.0001)
	};
	

	// /////////////// Do the tree
		
	// Setting the root cell. 
	// Remember - we add 7 to the advantage when we refer to a cell in the matrix, because indexes
	// start in 1 instead of -6
	mySet[orig_games + 1, orig_adv + 7] = 1
	EpointsLeft = 0
	
	// First, the probabilities to win the current game (unless it was a terminal node or the 6:6,
	//      because we deal with tiebreaker later)
	//  non-terminal nodes are either [5:(5 or less) or the same for numbers less than 5] or
	//  6:5, 5:6, 6:6. For the former, |adv| + games <= 10. For the latter (ignoring 6:6)
	//  it's max(me,opp)==6 && min(me,opp)==5.
	if (orig_games + abs(orig_adv) <= 10 || (max((myGames, oppGames)) == 6  & min((myGames,oppGames)) == 5)) {
		mySet[orig_games + 2, orig_adv + 6] = 1 - origGame.Pwin
		mySet[orig_games + 2, orig_adv + 8] = origGame.Pwin
		EpointsLeft = EpointsLeft + origGame.EpointsLeft
	}

	// Need this for the expectation (Pr[G<g])
	PrGameAlreadyEnded = 0 
	// Since we start from orig_games + 2, in cases where the function gets
	// a set score of a finished set (6:0, 6:1, ...) and the tree contains only one 1 and the
	// rest are zeroes, I need to put the probability of ending the set right after the current game
	// (orig_game+1)
	if (orig_games >= 6) {
		PrGameAlreadyEnded = mySet[orig_games+1 , -6 + orig_games+1] + ///
							mySet[orig_games+1 , 20 - orig_games-1]
	}
	
	// Starting the next game and until the tiebreak, all games look the same conditional on serving
	// or receiving. 
	// Thus, We will go in each line (number of games played) and for each node we will take the source node
	// from the previous line (the node from which we can arrive to this node) and multiply by the probability
	// to arrive to this point (either win or lose). From a (adv-1) advantage to adv, we need to win (to go from 
	// -1 to 0 or from 2 to 3) and from a (adv+1) advantage to adv, we need to lose (from 2 to 1 or -3 to -4).
	// When both are possible (to get to 2:1 we can either win after a 1:1 or lose after a 2:0), we sum the
	// probabilities because the events (score in the previous point) are disjoint.
	for(games = orig_games + 2 ; games <= 12 ; games++) {
		// Use the probabilities to win according to the server.
		// If the original points and current points have the same parity (even/odd)
		// then serving is the same
		if (mod(orig_games, 2) == mod(games-1, 2)) {
			PwinCurGame		= genGameServ.Pwin * serving + genGameRecv.Pwin * (1 - serving)
			EpointsCurGame	= genGameServ.EpointsLeft * serving + genGameRecv.EpointsLeft * (1 - serving)
		}
		else {
			PwinCurGame		= genGameServ.Pwin * (1 - serving) + genGameRecv.Pwin * serving
			EpointsCurGame	= genGameServ.EpointsLeft * (1 - serving) + genGameRecv.EpointsLeft * serving
		}
		
		for(adv=-6 ; adv <= 6 ; adv++) {
			// When do we have two sources for the same point?
			// (1) Whenever games + |adv| <= 10 & |adv| < games
			// (2) in 6:6 (actually, any x:x but 0:0 has two sources, but 0:0 is not in the loop)
			if (((games + abs(adv) <= 10) && (abs(adv) < games)) || ///
				(adv == 0)) {
				mySet[games+1, 7 + adv] = 	mySet[games , 7 + adv - 1] * PwinCurGame + ///
											mySet[games , 7 + adv + 1] * (1-PwinCurGame)
			}
			// When do we have one source for the same point?
			// (1) games + |adv| == 12 (except for 6:6 - adv == 0)
			// (2) games == |adv|  (upper diagonals)
			// (3) games >= 12 && |adv| == 2 (7:5 and 5:7)
			//       (6:7 and 7:6 are after a tiebreak, so they 
			//								will be dealt with outside the loop)
			// When there is only one source, it's determined by the adv:
			//   for positive adv, we won a previous game.
			//   for negative adv, we lost a previous game.
			else if ((abs(adv)+games == 12 && adv != 0) || ((games >= 12) && abs(adv) == 2) || abs(adv)==games) {
				if (adv > 0) {
					mySet[games+1, 7 + adv] = mySet[games , 7 + adv - 1] * PwinCurGame
				}
				else {
					mySet[games+1, 7 + adv] = mySet[games , 7 + adv + 1] * (1-PwinCurGame)
				}
			}
		
		}
		
		// We need to add the E[points left]
		// formula is sum_{g=orig_games^13} ( E(points_in_game) * Pr[G>=g] )
		// or alternatively: sum_{g=orig_games^13} ( E(points_in_game) * (1 - Pr[G<g]) )
		// First, update PrGameAlreadyEnded (add the probabilities to reach terminal node in
		// *previous* game).
		// if games == 12 then we can't have stopped the game in the 11th game so no need to update
		// if games < 6 then also there's no reason to stop the game yet.
		if ((games > 6) && (games <= 11)) {  	
			PrGameAlreadyEnded = PrGameAlreadyEnded + mySet[games , -6 + games] + mySet[games , 20 - games]
		};
		// Now, apply the formula.
		EpointsLeft = EpointsLeft + EpointsCurGame * (1 - PrGameAlreadyEnded)
	}
	
	// Tiebreak probabilities and expectation
	
	// We might have already got 7:6, so no need to update the 1 in the root and 0 in the rest
	// Thus, only if the sum of previous games is less than 13
	if (orig_games < 13) {
		mySet[14,8] = mySet[13,7] * tieBreak.Pwin
		mySet[14,6] = mySet[13,7] * (1 - tieBreak.Pwin)
		// this is what we're doing: EpointsLeft = EpointsLeft + E[points in TB]*Pr[6:6 in set]
		EpointsLeft = EpointsLeft + tieBreak.EpointsLeft * mySet[13,7]
	}
	
	// Calculate overall probability to win set
					//     6:0			  6:1			6:2				6:3
	probWinSet = 	mySet[7, 13] + mySet[8, 12] + mySet[9, 11] + mySet[10, 10] + ///
					mySet[11, 9] + mySet[13, 9] + mySet[14, 8]
					//		6:4				7:5			7:6

	// Calculate probability to switch serve.
	// First, if we started in a tiebreak, then we know that the side that
	// served the first point of the tiebreak will receive the next game (of the next set)
	// all we need to do is to know who served the first point of the tiebreak, not this point
	// of the tiebreak.
	if (myGames == 6 && oppGames == 6) {
		// As always, mod(tot_points_in_TB - 1, 4) == 0 or 1 then
		// whoever serving this point is not the one who served the first point of the 
		// TB, so s/he will definitely serve next game. Thus, this point's server will
		// not change
		if (mod(myPoints + oppPoints - 1, 4) <= 1) {
			returnMe.Pwin_n_serve = serving * mySet[14,8]
			returnMe.Plose_n_serve = serving * mySet[14,6]
			returnMe.Pwin_n_receive = (1-serving) * mySet[14,8]
			returnMe.Plose_n_receive = (1-serving) * mySet[14,6]
		}
		else {
			returnMe.Pwin_n_serve = (1-serving) * mySet[14,8]
			returnMe.Plose_n_serve = (1-serving) * mySet[14,6]
			returnMe.Pwin_n_receive = serving * mySet[14,8]
			returnMe.Plose_n_receive = serving * mySet[14,6]
		}
	}
	// Otherwise, probability to switch is the probability to have an odd number of 
	// games until the set ends.
	else {
			//			6:1				6:3				7:6		
		oddWin   = mySet[8, 12] + mySet[10, 10] + mySet[14, 8]
			//			1:6				3:6				6:7		
		oddLoss  = mySet[8, 2] + mySet[10, 4] + mySet[14, 6]
			//			6:0				6:2				6:4			7:5
		evenWin  = mySet[7, 13] + mySet[9, 11] + mySet[11, 9] + mySet[13, 9]
			//			0:6				2:6				4:6			5:7		
		evenLoss = mySet[7, 1] + mySet[9, 3]  + mySet[11, 5] + mySet[13, 5]
		
		if (mod(myGames + oppGames, 2) == 1) {
			returnMe.Pwin_n_serve 		= serving * oddWin + (1-serving) * evenWin
			returnMe.Pwin_n_receive 	= (1-serving) * oddWin + serving * evenWin
			returnMe.Plose_n_serve 		= serving * oddLoss + (1-serving) * evenLoss
			returnMe.Plose_n_receive 	= (1-serving) * oddLoss + serving * evenLoss
		}
		else {
			returnMe.Pwin_n_serve 		= (1-serving) * oddWin + serving * evenWin
			returnMe.Pwin_n_receive 	= serving * oddWin + (1-serving) * evenWin
			returnMe.Plose_n_serve 		= (1-serving) * oddLoss + serving * evenLoss
			returnMe.Plose_n_receive 	= serving * oddLoss + (1-serving) * evenLoss
		}
	}
	
	returnMe.probs = mySet
	returnMe.EpointsLeft = EpointsLeft
	returnMe.Pwin = probWinSet
	
	return(returnMe)
}


// OK, MATCH TREE
// A Match tree is different from the above trees because
// in each internal node of the match, we don't know if we are serving or receiving.
// Thus, for each internal node we'll have two nodes. One for Pr[playing this score ^ serving]
// while the other for Pr[playing this score ^ receiving].
// Final score-nodes will also be a pair, for generality. The actual winning probability for every
// final score is actually the sum of the two nodes

//       1   2   3   4   5   6   7   8   9   10
//    +-----------------------------------------+
//  1 |  .   .   .   .   0---0   .   .   .   .  |
//  2 |  .   .   0---0   .   .   0---0   .   .  |
//  3 |  0---0   .   .   0---0   .   .   0---0  |
//  4 |  .   .   0---0   .   .   0---0   .   .  |
//    +-----------------------------------------+

// The 0---0 represent score nodes. Left score-serving node is for 
// the probability to SERVE in this score, while the right score-serving
// node is probability to RECEIVE in this score.
// * Each of the score-serving node is linked to each of the score-serving nodes
//   in the adjacent scores nodes
// * The top score nodes (0:0) will always be either 0,0 or 1,1


struct matchStruct scalar function matchCalc(	real scalar mySets, real scalar oppSets, ///
							real scalar myGames, real scalar oppGames, ///
							real scalar myPoints, real scalar oppPoints, ///
							real scalar PwinPointServ, real scalar PwinPointRecv, ///
							real scalar serving, |real scalar debug) {
	struct setStruct scalar curSet, newSetServing, newSetReceiving
	struct matchStruct scalar returnMatch
	
	if (args() < 10) debug = 0;
		
	if (max((mySets,oppSets)) >= 2) {
		if (mySets > oppSets) {
			returnMatch.Pwin = 1
			return (returnMatch)
		} else if (mySets < oppSets) {
			returnMatch.Pwin = 0
			return (returnMatch)
		} else {
			printf("Error in entered set score\n\n");
			returnMatch.Pwin = .
			return (returnMatch)
		}
	}
	
	orig_sets = mySets + oppSets
	orig_adv  = mySets - oppSets
	match = J(4,10,0)
	// The root set should have 1,0 if serving or 0,1 if receiving
	match[orig_sets + 1 , orig_adv*2 + 5] = serving
	match[orig_sets + 1 , orig_adv*2 + 6] = (1-serving)

	curSet = setCalc(myGames, oppGames, myPoints, oppPoints, ///
				PwinPointServ, PwinPointRecv, serving, debug)
	
	if (curSet.Pwin == .) {
		returnMatch.Pwin = .
		return (returnMatch)
	}
	
	// DEBUG
	if (debug) {
		printf("Probability to win this set (%g:%g %g-%g) is %g\n", myGames, oppGames, myPoints, ///
				oppPoints, round(curSet.Pwin, 0.000001))
		round(curSet.probs, 0.0001)
	}
	
	match[orig_sets + 2 , orig_adv*2 + 3] = curSet.Plose_n_serve											
	match[orig_sets + 2 , orig_adv*2 + 4] = curSet.Plose_n_receive
	match[orig_sets + 2 , orig_adv*2 + 7] = curSet.Pwin_n_serve											
	match[orig_sets + 2 , orig_adv*2 + 8] = curSet.Pwin_n_receive
		
	if (orig_sets < 2) {
		newSetServing   = setCalc(0, 0, 0, 0, PwinPointServ, PwinPointRecv, 1, debug)
		newSetReceiving = setCalc(0, 0, 0, 0, PwinPointServ, PwinPointRecv, 0, debug)
		for (setNum = orig_sets + 1; setNum <= 2 ; setNum++) {
			for (adv = -1 ; adv <=1 ; adv++) {
				// Note that since we distribute probabilities from the upper row
				// to the one below, there will be more than one operation on each 
				// destination cell in the bottom row. This is why we always add
				// the previous value: a = a + something (instead of a = something)
				if (mod(adv,2)==mod(setNum,2)) {
					// lost, to serve from serve
					match[setNum + 2, adv*2 + 3] = match[setNum + 2, adv*2 + 3] + ///
						match[setNum + 1 , adv*2 + 5] * newSetServing.Plose_n_serve
					// lost, to serve from receive
					match[setNum + 2, adv*2 + 3] = match[setNum + 2, adv*2 + 3] + ///
						match[setNum + 1 , adv*2 + 6] * newSetReceiving.Plose_n_serve
					// lost, to receive from serve
					match[setNum + 2, adv*2 + 4] = match[setNum + 2, adv*2 + 4] + ///
						match[setNum + 1 , adv*2 + 5] * newSetServing.Plose_n_receive
					// lost, to receive from receive
					match[setNum + 2, adv*2 + 4] = match[setNum + 2, adv*2 + 4] + ///
						match[setNum + 1 , adv*2 + 6] * newSetReceiving.Plose_n_receive
					// won, to serve from serve
					match[setNum + 2, adv*2 + 7] = match[setNum + 2, adv*2 + 7] + ///
						match[setNum + 1 , adv*2 + 5] * newSetServing.Pwin_n_serve
					// won, to serve from receive
					match[setNum + 2, adv*2 + 7] = match[setNum + 2, adv*2 + 7] + ///
						match[setNum + 1 , adv*2 + 6] * newSetReceiving.Pwin_n_serve
					// won, to receive from serve
					match[setNum + 2, adv*2 + 8] = match[setNum + 2, adv*2 + 8] + ///
						match[setNum + 1 , adv*2 + 5] * newSetServing.Pwin_n_receive
					// won, to receive from receive
					match[setNum + 2, adv*2 + 8] = match[setNum + 2, adv*2 + 8] + ///
						match[setNum + 1 , adv*2 + 6] * newSetReceiving.Pwin_n_receive
				}
			}
		}
	}

	returnMatch.Pwin = match[3,9] + match[3,10] + match[4,7] + match[4,8]
	returnMatch.probs = match
	
	return (returnMatch)
}

real scalar function winMatchProb(real scalar mySets, real scalar oppSets, ///
						real scalar myGames, real scalar oppGames, ///
						real scalar myPoints, real scalar oppPoints, ///
						real scalar PwinServ, real scalar PwinRecv, ///
						real scalar serving, | real scalar debug) {
	struct matchStruct scalar matchResult
	
	if (args() < 10) debug = 0;
	
	matchResult = matchCalc(mySets, oppSets, myGames, oppGames, myPoints,
							oppPoints, PwinServ, PwinRecv, serving, debug)
	
	return(matchResult.Pwin)
}


real matrix function readSet(	real scalar myGames, real scalar oppGames, ///
						real scalar myPoints, real scalar oppPoints, ///
						real scalar PwinServ, real scalar PwinRecv, ///
						real scalar serving) {
	struct setStruct scalar setResult
	
	setResult = setCalc(myGames, oppGames, myPoints, oppPoints, PwinServ, PwinRecv, serving)
		
	round(setResult.probs,0.0001)
	setResult.Pwin
	setResult.EpointsLeft
	
	setResult.Pwin_n_serve
	setResult.Pwin_n_receive
	setResult.Plose_n_serve
	setResult.Plose_n_receive
	
	return (setResult.probs)
}

real scalar function EpointsLeft(real scalar myGames, real scalar oppGames, ///
						real scalar myPoints, real scalar oppPoints, ///
						real scalar PwinServ, real scalar PwinRecv, ///
						real scalar serving) {
	struct setStruct scalar setResult
	
	setResult = setCalc(myGames, oppGames, myPoints, oppPoints, PwinServ, PwinRecv, serving)
	return(setResult.EpointsLeft)
}

real matrix function winSetProb(	real scalar myGames, real scalar oppGames, ///
						real scalar myPoints, real scalar oppPoints, ///
						real scalar PwinServ, real scalar PwinRecv, ///
						real scalar serving) {
	struct setStruct scalar setResult
	
	setResult = setCalc(myGames, oppGames, myPoints, oppPoints, PwinServ, PwinRecv, serving)
	return(setResult.Pwin)
}

real matrix function readTiebreak(	real scalar myPoints, real scalar oppPoints, ///
							real scalar PwinServ, real scalar PwinRecv, ///
							real scalar serving) {
	struct gameStruct scalar gameResult
	
	gameResult = tiebreakCalc(myPoints, oppPoints, PwinServ, PwinRecv, serving)
	
	gameResult.Pwin
	gameResult.EpointsLeft
	
	return (gameResult.probs)
}


real matrix function readGame(	real scalar myPoints, real scalar oppPoints, ///
							real scalar Pwin) {
	struct gameStruct scalar gameResult
	
	gameResult = gameCalc(myPoints, oppPoints, Pwin)
	
	round(gameResult.probs,0.001)
	gameResult.Pwin
	gameResult.EpointsLeft
	
	return (gameResult.Pwin)
}

// FOR DEBUGGING
end
cap mata mata drop doObs()
mata
void function doObs(real scalar obsNum) {
	myGamesAllSets   = st_data(obsNum,tokens("set1_chal set2_chal set3_chal"))
	oppGamesAllSets  = st_data(obsNum,tokens("set1_opp set2_opp set3_opp"))
	curSetNum 		= st_data(obsNum,"set")
	myPoints  		= st_data(obsNum,"game_chal")
	oppPoints 		= st_data(obsNum,"game_opp")
	serving   		= st_data(obsNum,"chal_serving")
	probs     		= st_data(obsNum,tokens("mtch_chal_pt_win_prob_srv mtch_chal_pt_win_prob_rcv"))

	obs = 1
	dic = J(1,51,.)

	dic[51] = (dic[41] = (dic[31] = (dic[16] = (dic[1] = 0) + 1) + 1) + 1) + 1
	dic

	PWMs = J(1,3,.)
	
	// Calculate the number of sets I won and the number of sets opponent won
	errFlag = 0
	mySets = 0
	oppSets = 0
	for(setNum = 1 ; setNum < curSetNum[obs] ; setNum++) {
		if (myGamesAllSets[obs, setNum] == . | oppGamesAllSets[obs, setNum] == .) {
			printf("No game info for previous sets\n")
			errFlag = 1
		}
		else if (myGamesAllSets[obs, setNum] > oppGamesAllSets[obs, setNum]) {
			mySets++
		}
		else if (myGamesAllSets[obs, setNum] < oppGamesAllSets[obs, setNum]) {
			oppSets++
		}
		else {
			printf("Previous set(s) undetermined\n")
			errFlag = 1
		}
	}
		
	myGames  = myGamesAllSets[obs, curSetNum[obs]]
	oppGames = oppGamesAllSets[obs, curSetNum[obs]]
	
	servingLast = serving[obs]
	servingNext = servingLast
	
	// If it's a tiebreaker, next point might have a different server
	if (myGames == 6 & oppGames == 6) {
		if (mod(myPoints[obs] + oppPoints[obs], 2) == 0) {
			servingNext = 1 - servingLast
		}
		myPointsThisObs = myPoints[obs]
		oppPointsThisObs = oppPoints[obs]
	}
	else {
		// if it's not a tiebreaker, translate 15 to 1, 30 to 2, 40 to 3 and 50 to 4
		if (myPoints[obs] > 50)
			myPointsThisObs = .
		else
			myPointsThisObs = dic[myPoints[obs]+1]
		
		if (oppPoints[obs] > 50)
			oppPointsThisObs = .
		else
			oppPointsThisObs = dic[oppPoints[obs]+1]
	}
	
	printf("Current score is %g-%g (sets) %g:%g (games) %g-%g (points)\n",mySets, oppSets, myGames, oppGames, myPointsThisObs, oppPointsThisObs)
	printf("Serving is %g and probability to win a point is %g serving and %g receiving\n\n",servingLast, probs[obs, 1], probs[obs, 2])
	// Probability to win match if point challenged was won
	PWMs[obs, 1] = winMatchProb(mySets, oppSets, myGames, oppGames, ///
								myPointsThisObs+1, oppPointsThisObs, probs[obs, 1], ///
								probs[obs, 2], servingNext, 1)
								
	// Probability to win match if point challenged was lost
	PWMs[obs, 2] = winMatchProb(mySets, oppSets, myGames, oppGames, ///
								myPointsThisObs, oppPointsThisObs+1, probs[obs, 1], ///
								probs[obs, 2], servingNext, 1)
								
	// Probability to win match if point challenged was replayed
	PWMs[obs, 3] = winMatchProb(mySets, oppSets, myGames, oppGames, ///
								myPointsThisObs, oppPointsThisObs, probs[obs, 1], ///
								probs[obs, 2], servingLast, 1)
								
	PWMs[obs, .]
}


// Regular game means a game that starts in 0:0 (i.e not the current game in which it can be 2:1)
/*
Pwin_serv = 0.7
Pwin_recv = 0.45

t = readGame(3, 1, 0.5)
1-t


t_chalLost = readGame(3, 4, 0.5)
t_chalWon = readGame(4, 3, 0.5)
t_chalWon - t_chalLost

Ps = readSet(0, 0, 0, 0, Pwin_serv, Pwin_recv, 1)

Ps = readSet(6, 6, 1, 2, Pwin_serv, Pwin_recv, 0)

Ps = readSet(4, 5, 1, 3, Pwin_serv, Pwin_recv, 1)

Tb = readTiebreak(0, 0, Pwin_serv, Pwin_recv, 1)
Tb = readTiebreak(0, 0, Pwin_serv, Pwin_recv, 0)
*/

/* If you want to check why the probability to win a new set does not
   depend on who is serving

psame = J(10,10,.)
for (i = 0 ; i < 10 ; i++) {
for (j = 0 ; j < 10 ; j++) {
a = winSetProb(0,0,0,0,i/10,j/10,0)
b = winSetProb(0,0,0,0,i/10,j/10,1)
psame[i + 1, j + 1] = round(a - b, 0.0000001) == 0
}
}
psame
*/

end

