clear
set more off
set logtype text

cd "<SET YOUR FOLDER HERE>"

cap log close
log using "..\log\redForm.log", replace

use "..\Data\point_data_after_importance_round3"

count
// ONLY SINGLES MEN GAMES
keep if !doubles & !female

count

gen pwm_diff = (pwm_if_won - pwm_if_lst) * 100
gen chal_won_100 = challenge_won * 100

///////// GAMEPLAN ////////
// This file creates the following tables (as numbered in the paper (at least first draft)):
// 
//  1 - Comparison of scenarios within pairs.
//
// A1 - Comparison of summary statistics between the full sample and restricted sample
//      (full sample is all challenges that we have, restricted is the one for which we can get a 
//       valid tennis state (valid score, serving status and challenges-left)).
// 
//  2 - Importance measure (and option value measure) for different scenarios
//
//  3 - Regressions of importance measure and option value measures.

// Change distance_mm so that wrong challenges will be negative
gen distance_mm_won = challenge_won * distance_mm - (1-challenge_won) * distance_mm

// chals_in_match counts how many challenges there are in each match
egen chals_in_match = count(match_id), by(match_id)

// Generate ln(rank) from rank
gen ln_chal_rank = ln(challenge_maker_rank1)
gen ln_opp_rank  = ln(opponent_rank1)

// Defining scenarios (situations) to compare
gen 	scenario1 = 1 if set == 1 | set == 2
replace scenario1 = 0 if set == 3
label variable scenario1 "first or second sets (1) vs. third set (0)"

gen		scenario2 = 1 if max(cur_set_chal, cur_set_opp) < 4
replace scenario2 = 0 if max(cur_set_chal, cur_set_opp) >= 4 & cur_set_chal != . & cur_set_opp != .
label variable scenario2 "early in set [game_chal,game_opp]<[4,4] (1) vs. late in set (0)"

gen 	scenario3 = 0 if game_chal == 40 & game_opp == 40
replace scenario3 = 1 if (game_chal == 40 & game_opp == 0) |  ///
						(game_chal == 0 & game_opp == 40)
label variable scenario3 "game score 0-40 or 40-0 (1) vs. game score 40-40 (0)"

gen 	scenario4 = 1-tiebreak
label variable scenario4 "non-tiebreak (1) vs. tiebreak (0)"

gen 	scenario5 = 0 if challenges_left > 1
replace scenario5 = 1 if challenges_left == 1
label variable scenario5 "1 challenge left (1) vs. more than one challenges left (0)"

// local controls "ln_chal_rank ln_opp_rank match_time call_in tiebreak"

local restrictedSampleCond "pwm_diff != ."
local unrestrictedSampleCond "1==1"

cap postclose table1_restricted
cap postclose table1_unrestricted
cap postclose table2

postfile table1_restricted str80 scenario_pair scenario obs perc_won pwon_difference pwon_difference_se pwon_diff_pval mean_dist dist_difference dist_difference_se dist_diff_pval ///
	using "..\out\new_NEWtable1", replace
postfile table1_unrestricted str80 scenario_pair scenario obs perc_won pwon_difference pwon_difference_se pwon_diff_pval mean_dist dist_difference dist_difference_se dist_diff_pval ///
	using "..\out\new_NEWtableA1", replace
postfile table2 str80 scenario_pair scenario obs space1 imp_mean imp_sd imp_10p imp_90p space2 opt_mean opt_sd opt_10p opt_90p ///
	using "..\out\new_NEWtable2", replace

su pwm_diff if distance_mm_won != ., det
local obs      = r(N)
local imp_mean = r(mean)
local imp_sd   = r(sd)
local imp_p10  = r(p10)
local imp_p90  = r(p90)

su epl_if_lst if distance_mm_won != ., det
local opt_mean = r(mean)
local opt_sd   = r(sd)
local opt_p10  = r(p10)
local opt_p90  = r(p90)

post table2 ("all sample") (.) (`obs') (.) (`imp_mean') (`imp_sd') (`imp_p10') (`imp_p90') (.) ///
						                   (`opt_mean') (`opt_sd') (`opt_p10') (`opt_p90')

forvalues scen = 1/5 {
	local description : variable label scenario`scen'
	
	forvalues scen_specifically = 1(-1)0 {
		if (`scen' <= 4) {
			foreach vers in restricted unrestricted {
				di as input " "
				di "su chal_won_100 if scenario`scen' == `scen_specifically' & ``vers'SampleCond'"
				su chal_won_100 if scenario`scen' == `scen_specifically' & ``vers'SampleCond'
				local mean_won = r(mean)
				
				di as input "reg chal_won_100 scenario`scen' if ``vers'SampleCond'"
				reg chal_won_100 scenario`scen' if ``vers'SampleCond'
					
				local pwon_diff_b = _b[scenario`scen']
				local pwon_diff_se = _se[scenario`scen']
				local pwon_diff_pval = ttail(e(df_r),abs(`pwon_diff_b' / `pwon_diff_se'))*2

				di as input " "
				di "su distance_mm_won if scenario`scen' == `scen_specifically' & ``vers'SampleCond'"
				su distance_mm_won if scenario`scen' == `scen_specifically' & ``vers'SampleCond'
				local mean_dist = r(mean)
				local obs       = r(N)
				
				di as input "reg distance_mm_won scenario`scen' if ``vers'SampleCond'"
				reg distance_mm_won scenario`scen' if ``vers'SampleCond'
					
				local dist_diff_b = _b[scenario`scen']
				local dist_diff_se = _se[scenario`scen']
				local dist_diff_pval = ttail(e(df_r),abs(`dist_diff_b' / `dist_diff_se'))*2
				
				post table1_`vers' ("`description'") (`scen_specifically') (`obs') (`mean_won') (`pwon_diff_b') (`pwon_diff_se') (`pwon_diff_pval') (`mean_dist') (`dist_diff_b') (`dist_diff_se') (`dist_diff_pval')
			}
		}
		su pwm_diff if scenario`scen' == `scen_specifically' & distance_mm_won != ., det
		local obs      = r(N)
		local imp_mean = r(mean)
		local imp_sd   = r(sd)
		local imp_p10  = r(p10)
		local imp_p90  = r(p90)
		
		su epl_if_lst if scenario`scen' == `scen_specifically' & distance_mm_won != ., det
		local opt_mean = r(mean)
		local opt_sd   = r(sd)
		local opt_p10  = r(p10)
		local opt_p90  = r(p90)

		post table2 ("`description'") (`scen_specifically') (`obs') (.) (`imp_mean') (`imp_sd') (`imp_p10') (`imp_p90') (.) ///
																		(`opt_mean') (`opt_sd') (`opt_p10') (`opt_p90')
	}
}

postclose table1_restricted
postclose table1_unrestricted
postclose table2

preserve
use "..\out\new_NEWtable1", clear
outsheet using "..\out\new_NEWtable1.csv", comma replace

use "..\out\new_NEWtableA1", clear
outsheet using "..\out\new_NEWtableA1.csv", comma replace

use "..\out\new_NEWtable2", clear
outsheet using "..\out\new_NEWtable2.csv", comma replace

restore

cap erase "..\out\tableNEW3.xml"
cap erase "..\out\tableNEW3.txt"

// TABLE 3 - Regressions with importance and option measures
local controls   "ln_chal_rank ln_opp_rank chal_serving player_stopped match_time call_in"
xi i.set

foreach depvar in chal_won_100 distance_mm_won {
	di " "
	di as input "Regressions for `depvar' with depvar in chal_won_100 distance_mm_won"
	di " "
	su `depvar' if pwm_diff != . & challenges_left != . & epl_if_lst != . & distance_mm != .
	
	reg `depvar' pwm_diff challenges_left epl_if_lst if distance_mm != ., cluster(match_id)
	outreg2 pwm_diff challenges_left epl_if_lst using "..\out\tableNEW3.xml", excel auto(8)
	
	reg `depvar' pwm_diff challenges_left epl_if_lst `controls' if distance_mm != ., cluster(match_id)
	outreg2 pwm_diff challenges_left epl_if_lst using "..\out\tableNEW3.xml", excel auto(8)
	
	xtreg `depvar' pwm_diff challenges_left epl_if_lst `controls' if distance_mm != ., i(match_id) fe cluster(match_id) 
	outreg2 pwm_diff challenges_left epl_if_lst using "..\out\tableNEW3.xml", excel auto(8)
	
	reg `depvar' pwm_diff challenges_left epl_if_lst `controls' _Iset* if distance_mm != ., cluster(match_id)
	outreg2 pwm_diff challenges_left epl_if_lst using "..\out\tableNEW3.xml", excel auto(8)
	
	xtreg `depvar' pwm_diff challenges_left epl_if_lst `controls' _Iset* if distance_mm != ., i(match_id) fe cluster(match_id) 
	outreg2 pwm_diff challenges_left epl_if_lst using "..\out\tableNEW3.xml", excel auto(8)
}
